/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

//
// Created by hxf on 22-8-25.
//

#include "layout-config.h"
#include "global-settings.h"
#include "screen-monitor.h"

#include <QDebug>
#include <QPushButton>
#include <QtMath>

namespace Sidebar {

static LayoutConfig *globalLayoutInstance = nullptr;

LayoutConfig *LayoutConfig::getInstance(QObject *parent)
{
    if (!globalLayoutInstance) {
        globalLayoutInstance = new LayoutConfig(parent);
    }

    return globalLayoutInstance;
}

LayoutConfig::LayoutConfig(QObject *parent) : QObject(parent)
{
    //平板模式布局作为默认布局
    loadTabletLayout();

    m_isTabletMode = GlobalSettings::globalInstance()->getValue(TABLET_MODE).toBool();
    if (!m_isTabletMode) {
        loadPCLayout();
    }

//    //测试布局切换功能
//    bool *isPC = new bool;
//    *isPC = !m_isTabletMode;

//    QPushButton *button = new QPushButton();
//    button->setText("点击切换");
//    button->resize(200, 200);

//    connect(button, &QPushButton::clicked, this, [=] {
//        if (*isPC) {
//            *isPC = false;
//            loadTabletLayout();

//        } else {
//            *isPC = true;
//            loadPCLayout();
//        }

//        Q_EMIT modelChanged(!(*isPC));
//    });

//    button->show();
}

bool LayoutConfig::isTabletMode()
{
    return m_isTabletMode;
}

LayoutConfig::~LayoutConfig()
{

}

void LayoutConfig::updateLayout(bool isTabletMode)
{
    if (isTabletMode == m_isTabletMode) {
        return;
    }

    m_isTabletMode = isTabletMode;
    Q_EMIT systemModeChanged(m_isTabletMode);

    if (m_isTabletMode) {
        loadTabletLayout();
    } else {
        loadPCLayout();
    }
}

/**
 * 修改布局为pc模式的布局，只需要修改跟平板模式不同的部分即可
 */
void LayoutConfig::loadPCLayout()
{
    // 快捷操作主体
    m_width.insert(LayoutComponent::SidebarMain, PC_SIDEBAR_WIDTH);
    m_spacing.insert(LayoutComponent::SidebarMain, 0);

    // UserInfo
    m_height.insert(LayoutComponent::SidebarUserInfo, 68);

    m_radius.insert(LayoutComponent::SidebarUserInfo, 0);
    m_spacing.insert(LayoutComponent::SidebarUserInfo, 8);
    m_margin.insert(LayoutComponent::SidebarUserInfo, {24, 0, 24, 0});

    m_width.insert(LayoutComponent::SidebarUserInfoIcon, 48);
    m_height.insert(LayoutComponent::SidebarUserInfoIcon, 48);

    m_width.insert(LayoutComponent::SidebarUserInfoPowerIcon, 48);
    m_height.insert(LayoutComponent::SidebarUserInfoPowerIcon, 48);

    // MenuButton
    m_height.insert(LayoutComponent::SidebarMenuButton, 0);
    m_height.insert(LayoutComponent::SidebarMenuButtonView, 0);

    // IconButton
    // 宽度高度随着容器改变
    m_spacing.insert(LayoutComponent::SidebarIconButton, 1);
    // 高度与宽度一致
    m_width.insert(LayoutComponent::SidebarIconButtonIcon, 48);
    m_height.insert(LayoutComponent::SidebarIconButtonView, 206);
    m_radius.insert(LayoutComponent::SidebarIconButtonView, 0);
    m_margin.insert(LayoutComponent::SidebarIconButtonView, {0, 24, 0, 24});

    // ProgressBar
    m_height.insert(LayoutComponent::SidebarProgressBar, 40);
    m_radius.insert(LayoutComponent::SidebarProgressBar, 0);
    m_spacing.insert(LayoutComponent::SidebarProgressBar, 8);
    m_margin.insert(LayoutComponent::SidebarProgressBar, {24, 0, 24, 0});

    m_width.insert(LayoutComponent::SidebarProgressBarIcon, 16);
    m_height.insert(LayoutComponent::SidebarProgressBarIcon, 16);

    m_height.insert(LayoutComponent::SidebarProgressBarSlider, 16);
    m_radius.insert(LayoutComponent::SidebarProgressBarSlider, 8);

    m_spacing.insert(LayoutComponent::SidebarProgressBarView, 0);

    // SidebarWeather
    m_height.insert(LayoutComponent::SidebarWeather, 32);
    m_spacing.insert(LayoutComponent::SidebarWeather, 8);
    m_margin.insert(LayoutComponent::SidebarWeather, {24, 0, 24, 0});

    // SidebarFoldShortcutArea
    m_height.insert(LayoutComponent::SidebarFoldShortcutArea, 24);
    m_radius.insert(LayoutComponent::SidebarFoldShortcutArea, 12);

    m_width.insert(LayoutComponent::NotificationItem, 0);
    m_height.insert(LayoutComponent::NotificationItem, 108);
    m_radius.insert(LayoutComponent::NotificationItem, 6);
    m_spacing.insert(LayoutComponent::NotificationItem, 0);
    m_margin.insert(LayoutComponent::NotificationItem, {16, 16, 16, 4});

    m_height.insert(LayoutComponent::NotificationItemNormal, 20);
    m_height.insert(LayoutComponent::NotificationItemBody, 24);


    Q_EMIT layoutChanged();
}

void LayoutConfig::loadTabletLayout()
{
    qreal r = ScreenMonitor::getInstance()->screenRatio();
    // 快捷操作主体
    m_width.insert(LayoutComponent::SidebarMain, TABLET_SIDEBAR_WIDTH);
    m_spacing.insert(LayoutComponent::SidebarMain, qCeil(16*r));

    // UserInfo
    m_height.insert(LayoutComponent::SidebarUserInfo, qCeil(128*r));
    m_radius.insert(LayoutComponent::SidebarUserInfo, qCeil(32*r));
    m_spacing.insert(LayoutComponent::SidebarUserInfo, qCeil(8*r));
    m_margin.insert(LayoutComponent::SidebarUserInfo, {qCeil(32*r), 0, qCeil(32*r), 0});

    m_width.insert(LayoutComponent::SidebarUserInfoIcon, qCeil(80*r));
    m_height.insert(LayoutComponent::SidebarUserInfoIcon, qCeil(80*r));

    m_width.insert(LayoutComponent::SidebarUserInfoPowerIcon, qCeil(64*r));
    m_height.insert(LayoutComponent::SidebarUserInfoPowerIcon, qCeil(64*r));

    // MenuButton
    m_height.insert(LayoutComponent::SidebarMenuButton, qCeil(128*r));
    m_radius.insert(LayoutComponent::SidebarMenuButton, qCeil(32*r));
    m_spacing.insert(LayoutComponent::SidebarMenuButton, qCeil(8*r));
    m_margin.insert(LayoutComponent::SidebarMenuButton, {qCeil(32*r), 0, qCeil(8*r), 0});

    m_height.insert(LayoutComponent::SidebarMenuButtonView, qCeil(128*r));
    m_spacing.insert(LayoutComponent::SidebarMenuButtonView, qCeil(16*r));

    // IconButton
    // 宽度高度随着容器改变
    m_spacing.insert(LayoutComponent::SidebarIconButton, qCeil(8*r));
    // 高度与宽度一致
    m_width.insert(LayoutComponent::SidebarIconButtonIcon, qCeil(64*r));

    m_height.insert(LayoutComponent::SidebarIconButtonView, qCeil(256*r));
    m_radius.insert(LayoutComponent::SidebarIconButtonView, qCeil(32*r));
    m_margin.insert(LayoutComponent::SidebarIconButtonView, {0, qCeil(24*r), 0, qCeil(24*r)});

    // ProgressBar
    m_height.insert(LayoutComponent::SidebarProgressBar, qCeil(112*r));
    m_radius.insert(LayoutComponent::SidebarProgressBar, qCeil(32*r));
    m_spacing.insert(LayoutComponent::SidebarProgressBar, qCeil(16*r));
    m_margin.insert(LayoutComponent::SidebarProgressBar, {qCeil(32*r), 0, qCeil(32*r), 0});

    m_width.insert(LayoutComponent::SidebarProgressBarIcon, qCeil(32*r));
    m_height.insert(LayoutComponent::SidebarProgressBarIcon, qCeil(32*r));

    m_height.insert(LayoutComponent::SidebarProgressBarSlider, qCeil(48*r));
    m_radius.insert(LayoutComponent::SidebarProgressBarSlider, qCeil(16*r));

    m_spacing.insert(LayoutComponent::SidebarProgressBarView, qCeil(16*r));

    // SidebarWeather
    m_height.insert(LayoutComponent::SidebarWeather, 0);

    // SidebarFoldShortcutArea
    m_height.insert(LayoutComponent::SidebarFoldShortcutArea, 0);

    Q_EMIT layoutChanged();
}

int LayoutConfig::width(LayoutComponent::Component component)
{
    if (m_width.contains(component)) {
        return m_width.value(component);
    }

    return 0;
}

int LayoutConfig::height(LayoutComponent::Component component)
{
    if (m_height.contains(component)) {
        return m_height.value(component);
    }

    return 0;
}

int LayoutConfig::radius(LayoutComponent::Component component)
{
    if (m_radius.contains(component)) {
        return m_radius.value(component);
    }

    return 0;
}

int LayoutConfig::spacing(LayoutComponent::Component component)
{
    if (m_spacing.contains(component)) {
        return m_spacing.value(component);
    }

    return 0;
}

int LayoutConfig::margin(LayoutComponent::Component component, int position)
{
    if (m_margin.contains(component)) {
        switch (position) {
            case 0: return m_margin.value(component).left;
            case 1: return m_margin.value(component).top;
            case 2: return m_margin.value(component).right;
            case 3: return m_margin.value(component).bottom;
            default:
                break;
        }
    }

    return 0;
}

LayoutHelper::LayoutHelper(QObject *parent) : QObject(parent)
{
    m_layoutConfig = LayoutConfig::getInstance();
    if (m_layoutConfig) {
        connect(m_layoutConfig, &LayoutConfig::layoutChanged, this, &LayoutHelper::layoutChanged);
    }
}

void LayoutHelper::setComponent(LayoutComponent::Component component)
{
    m_component = component;
    Q_EMIT componentChanged();
    Q_EMIT layoutChanged();
}

LayoutComponent::Component LayoutHelper::getComponent()
{
    return m_component;
}

int LayoutHelper::getWidth()
{
    return m_layoutConfig->width(m_component);
}

int LayoutHelper::getHeight()
{
    return m_layoutConfig->height(m_component);
}

int LayoutHelper::getRadius()
{
    return m_layoutConfig->radius(m_component);
}

int LayoutHelper::getSpacing()
{
    return m_layoutConfig->spacing(m_component);
}

int LayoutHelper::getLeftMargin()
{
    return m_layoutConfig->margin(m_component, 0);
}

int LayoutHelper::getTopMargin()
{
    return m_layoutConfig->margin(m_component, 1);
}

int LayoutHelper::getRightMargin()
{
    return m_layoutConfig->margin(m_component, 2);
}

int LayoutHelper::getBottomMargin()
{
    return m_layoutConfig->margin(m_component, 3);
}

} // UkuiShortcut
